-------------------------------------------------------------------------------
-- ElvUI Equipment Sets Datatext By Lockslap
-------------------------------------------------------------------------------
E, _, V, P, G = unpack(ElvUI)
local DT = E:GetModule("DataTexts")
local L = LibStub("AceLocale-3.0"):GetLocale("ElvUI_EquipmentSetsDT", false)
local EP = LibStub("LibElvUIPlugin-1.0")
local AT = LibStub("AceTimer-3.0")

local join = string.join
local wipe = table.wipe
local sort = table.sort
local pairs = pairs

local displayString = ""
local chatString = ""
local iconString = "|T%s:14:14:0:0:64:64:4:60:4:60|t"
local hexColor = "ffffff"
local lastPanel

-- for drop down menu
local menuFrame = CreateFrame("Frame", "ESDTEquipmentSetMenu", E.UIParent, "UIDropDownMenuTemplate")

-- for renaming the equipment set
StaticPopupDialogs["ESDT_RENAME"] = {
	text = L["Rename %s to what?"],
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = true,
	maxLetters = 16,
	exclusive = 0,
	preferredIndex = 3,
	timeout = 0, 
	whileDead = true,
	hideOnEscape = true,
	OnShow = function(self) _G[self:GetName() .. "EditBox"]:SetFocus(); self:GetParent().button1:Disable() end,
	OnHide = function(self)
		if _G[self:GetName() .. "EditBox"]:IsShown() then
			_G[self:GetName() .. "EditBox"]:SetFocus()
		end
		_G[self:GetName() .. "EditBox"]:SetText("")
	end,
	OnAccept = function(self, oldName)
		local newName = _G[self:GetName() .. "EditBox"]:GetText()
		if not newName or newName == "" or oldName == newName then return end
		ModifyEquipmentSet(oldName, newName)
		DEFAULT_CHAT_FRAME:AddMessage(chatString:format((L["Renamed |cff%s%s|r to |cff%s%s|r!"]):format(hexColor, oldName, hexColor, newName)))
	end,
	EditBoxOnEnterPressed = function(self, oldName)
		local newName = _G[self:GetName() .. "EditBox"]:GetText()
		if not newName or newName == "" or oldName == newName then return end
		ModifyEquipmentSet(oldName, newName)
		DEFAULT_CHAT_FRAME:AddMessage(chatString:format((L["Renamed |cff%s%s|r to |cff%s%s|r!"]):format(hexColor, oldName, hexColor, newName)))
	end,
	EditBoxOnEscapePressed = function(self) self:GetParent():Hide() end,
	EditBoxOnTextChanged = function(self) self:GetParent().button1:Enable() end,
}

-- staticpoup for deleting
StaticPopupDialogs["ESDT_DELETE"] = {
	text = L["Are you sure you want to delete %s?"],
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	timeout = 10,
	whileDead = true,
	hideOnEscape = true,
	OnAccept = function(self, name)
		DeleteEquipmentSet(name)
		DEFAULT_CHAT_FRAME:AddMessage(chatString:format((L["Deleted equipment set |cff%s%s|r!"]):format(hexColor, name)))
	end,
}

local function SendDebugMessage(message)
	DEFAULT_CHAT_FRAME:AddMessage(chatString:format(message))
end

local function GetEquippedSet()
	local num = GetNumEquipmentSets()
	if num == 0 then return false end
	for i = 1, num do
		local name, icon, _, isEquipped = GetEquipmentSetInfo(i)
		if isEquipped == true then
			if E.db.equipsetsdt.debug == true then
				SendDebugMessage((L["Current equipment set is |cff%s%s|r."]):format(hexColor, name))
			end
			return name, icon
		end
	end
	return false
end

local function EquipmentSetClick(self, info)
	if not IsShiftKeyDown() and not IsControlKeyDown() and not IsAltKeyDown() then
		-- change set
		UseEquipmentSet(info)
		if lastPanel ~= nil then
			local icon = GetEquipmentSetInfoByName(info)
			lastPanel.text:SetFormattedText(displayString, E.db.equipsetsdt.dtIcon == true and iconString:format(("Interface\\Icons\\%s"):format(icon)) or "", info)
		end
		if E.db.equipsetsdt.debug == true then
			SendDebugMessage((L["Equipped |cff%s%s|r Set"]):format(hexColor, info))
		end
	elseif IsShiftKeyDown() then
		-- rename set
		local popup = StaticPopup_Show("ESDT_RENAME", info)
		if popup then
			popup.data = info
		end
	elseif IsControlKeyDown() then
		-- save set
		SaveEquipmentSet(info)
		SendDebugMessage((L["Saved |cff%s%s|r!"]):format(hexColor, info))
	elseif IsAltKeyDown() then
		-- delete set
		local popup = StaticPopup_Show("ESDT_DELETE", info)
		if popup then
			popup.data = info
		end
	end
end

local function OnClick(self, button)
	if button == "LeftButton" then
		-- wish there was a cleaner way to do this
		ToggleCharacter("PaperDollFrame")
		PaperDollSidebarTab3:Click()
		if E.db.equipsetsdt.debug == true then
			SendDebugMessage(L["Toggling Equipment Manager"])
		end
	elseif button == "RightButton" then
		DT.tooltip:Hide()
		
		local menuList = {{text = L["Choose Equipment Set"], isTitle = true, notCheckable = true,},}
		local numSets, curNumSets = GetNumEquipmentSets(), 2
		local color = "ffffff"
		
		if numSets == 0 then
			menuList[curNumSets] = {text = ("|cffff0000%s|r"):format(L["No Equipment Sets"]), notCheckable = true,}
		else
			for i = 1, numSets do
				local name, _, _, isEquipped, _, _, _, missing, _ = GetEquipmentSetInfo(i)
				
				if missing > 0 then
					color = "ff0000"
				else
					color = isEquipped == true and hexColor or "ffffff"
				end
				
				menuList[curNumSets] = {text = ("|cff%s%s|r"):format(color, name), func = EquipmentSetClick, arg1 = name, checked = isEquipped == true and true or false,}
				curNumSets = curNumSets + 1
			end
			
			-- add a hint
			menuList[curNumSets] = {text = L["Shift + Click to Rename"], isTitle = true, notCheckable = true, notClickable = true}
			menuList[curNumSets + 1] = {text = L["Ctrl + Click to Save"], isTitle = true, notCheckable = true, notClickable = true}
			menuList[curNumSets + 2] = {text = L["Alt + Click to Delete"], isTitle = true, notCheckable = true, notClickable = true}
		end
		EasyMenu(menuList, menuFrame, "cursor", 0, 0, "MENU", 2)
		
		if E.db.equipsetsdt.debug == true then
			SendDebugMessage(L["Showing drop down menu."])
		end
	end
end

local function OnEnter(self)
	local num = GetNumEquipmentSets()
	local color = "ffffff"
	local lineString = "%s |cff%s%s|r"
	local itemString = "|cff00ff00%d|r|cffffffff/|r|cff%s%d|r|cffffffff/|r|cff0000ff%d|r|cffffffff/|r|cffff0000%d|r"
	DT:SetupTooltip(self)
	
	if num == 0 then
		DT.tooltip:AddLine(displayString:format("", L["No Equipment Sets"]))
	else
		for i = 1, num do
			local name, icon, _, isEquipped, items, equipped, _, missing, ignored = GetEquipmentSetInfo(i)
			if isEquipped then
				color = hexColor
			else
				color = missing > 0 and "ff0000" or "ffffff"
			end
			DT.tooltip:AddDoubleLine(lineString:format(E.db.equipsetsdt.ttIcon == true and iconString:format(icon) or "", color, name), itemString:format(equipped, hexColor, items, ignored, missing))
		end
	end
	
	if E.db.equipsetsdt.hint == true then
		DT.tooltip:AddLine(" ")
		DT.tooltip:AddDoubleLine(("|cff00ff00%s|r"):format(L["Green Text"]), L["Equipped Items"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(("|cff%s%s|r"):format(hexColor, L["Color Text"]), L["Total Items"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(("|cff0000ff%s|r"):format(L["Blue Text"]), L["Ignored Items"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(("|cffff0000%s|r"):format(L["Red Text"]), L["Missing Items"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(L["Left Click"], L["Toggle Equipment Sets UI"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(L["Right Click"], L["Choose Equipment Set"], 1, 1, 1, 1, 1, 0)
	end
	DT.tooltip:Show()
end

local function OnEvent(self, event)
	local set, icon = GetEquippedSet()
	lastPanel = self
	if not set then
		self.text:SetFormattedText(displayString, "", L["No Equipped Set"])
		return
	end
	self.text:SetFormattedText(displayString, E.db.equipsetsdt.dtIcon == true and iconString:format(icon) or "", set)
end

P["equipsetsdt"] = {
	["dtIcon"] = false,
	["ttIcon"] = true,
	["hint"] = true,
	["debug"] = false,
}

local function InjectOptions()
	if not E.Options.args.lockslap then
		E.Options.args.lockslap = {
			type = "group",
			order = -2,
			name = L["Plugins by |cff9382c9Lockslap|r"],
			args = {
				thanks = {
					type = "description",
					order = 1,
					name = L["Thanks for using and supporting my work!  -- |cff9382c9Lockslap|r\n\n|cffff0000If you find any bugs, or have any suggestions for any of my addons, please open a ticket at that particular addon's page on CurseForge."],
				},
			},
		}
	elseif not E.Options.args.lockslap.args.thanks then
		E.Options.args.lockslap.args.thanks = {
			type = "description",
			order = 1,
			name = L["Thanks for using and supporting my work!  -- |cff9382c9Lockslap|r\n\n|cffff0000If you find any bugs, or have any suggestions for any of my addons, please open a ticket at that particular addon's page on CurseForge."],
		}
	end
	
	E.Options.args.lockslap.args.equipsetsdt = {
		type = "group",
		name = L["Equipment Sets Datatext"],
		get = function(info) return E.db.equipsetsdt[info[#info]] end,
		set = function(info, value) E.db.equipsetsdt[info[#info]] = value; DT:LoadDataTexts() end,
		args = {
			dtIcon = {
				type = "toggle",
				order = 1,
				name = L["Datatext Icon"],
				desc = L["Display the currently equipped set's icon on the datatext, if available."],
			},
			ttIcon = {
				type = "toggle",
				order = 2,
				name = L["Tooltip Icons"],
				desc = L["Display the icons for the equipment sets in the tooltip, if available."],
			},
			hint = {
				type = "toggle",
				order = 3,
				name = L["Show Hint"],
				desc = L["Show the hint in the tooltip."],
			},
			debug = {
				type = "toggle",
				order = 4,
				name = L["Debug Mode"],
				desc = L["Toggle debug mode.  |cffff0000Useful if the addon is not functioning properly.|r"],
			},
		},
	}
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = join("", "|cffffffffSet:|r %s ", hex, "%s|r")
	chatString = join("", hex, "ElvUI_ESDT|cffffffff: %s|r")
	hexColor = ("%02x%02x%02x"):format(r * 255, g * 255, b * 255) or "ffffff"
	
	if lastPanel ~= nil then
		OnEvent(lastPanel)
	end
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

EP:RegisterPlugin(..., InjectOptions)
DT:RegisterDatatext("Equipment Sets", {"PLAYER_ENTERING_WORLD", "WEAR_EQUIPMENT_SET", "EQUIPMENT_SETS_CHANGED", "ACTIVE_TALENT_GROUP_CHANGED", "EQUIPMENT_SWAP_FINISHED"}, OnEvent, nil, OnClick, OnEnter)